﻿Imports System.Text
Imports System.Text.RegularExpressions
Imports Newtonsoft.Json
Imports Newtonsoft.Json.Linq

Public Class CnslFns
    Shared config As String = IO.File.ReadAllText("FunctionKeys.json")
    Public Shared functionKeys As Dictionary(Of String, String) = JsonConvert.DeserializeObject(Of Dictionary(Of String, String))(config)
    Public Shared Function IsNotAFunctKey(s As String) As Boolean
        Return s.Length = 0 OrElse AscW(s) <> 27
    End Function
    Public Shared Function IsAFunctKey(s As String) As Boolean
        Return Not IsNotAFunctKey(s)
    End Function
End Class

Public Delegate Function CnslCallback(ByRef input As StringBuilder, objVal As CnslValidation) As Boolean
Public Class CnslValidation
    Friend inpType As CnslValTypes
    Friend callBk As CnslCallback
    Friend Shared CI As Globalization.CultureInfo = New Globalization.CultureInfo("en-US")
    Dim patt As String
    Dim dspMsg As Boolean
    Friend Shared errMessages As Dictionary(Of String, String)
    Friend Shared errMessage As String
    Dim maxTries As Int32 = 25
    Dim minLen As Int32 = 0

    Shared Sub New()
        Try
            Dim sLang As String = IO.File.ReadAllText("./lang/" + CI.Name + ".JSON")
            errMessages = JsonConvert.DeserializeObject(Of Dictionary(Of String, String))(sLang)


        Catch ex As Exception
        End Try
    End Sub
    Public Sub New(validationType As CnslValTypes,
                   displayErrMsg As Boolean,
                   Optional minLength As Int32 = 1,
                   Optional maxNumOfTries As Int32 = 25,
                   Optional regexPattern As String = "",
                   Optional cullture As Globalization.CultureInfo = Nothing,
                   Optional callback As CnslCallback = Nothing)
        inpType = validationType
        dspMsg = displayErrMsg
        minLen = minLength
        maxTries = maxNumOfTries
        patt = regexPattern
        If cullture IsNot Nothing Then
            CI = cullture
        End If
        callBk = callback
    End Sub
    Public Shared Property Culture As Globalization.CultureInfo
        Get
            Return CI
        End Get
        Set(value As Globalization.CultureInfo)
            CI = value
        End Set
    End Property
    Public Property ValidationType As CnslValTypes
        Get
            Return inpType
        End Get
        Set(value As CnslValTypes)
            inpType = value
        End Set
    End Property
    Public ReadOnly Property sDspErrMsg As Boolean
        Get
            Return errMessage
        End Get
    End Property
    Public Property minLength As Int32
        Get
            Return minLen
        End Get
        Set(value As Int32)
            minLen = value
        End Set
    End Property
    Public Property maxNumOfTries As Int32
        Get
            Return maxTries
        End Get
        Set(value As Int32)
            maxTries = value
        End Set
    End Property
    Public Function Validate(sb As StringBuilder) As Boolean
        Dim ret As Boolean = False
        Try
            errMessage = ""
            If sb.Length < minLen Then
                errMessage = GetMessage(1, dspMsg, minLen) ' "Input should be at least {0} characters long."
                Exit Try
            End If
            Select Case inpType
                Case CnslValTypes.Boolean
                    Dim bool As Boolean
                    If Not Boolean.TryParse(sb.ToString, bool) Then
                        errMessage = GetMessage(2, dspMsg) ' "Value should be True or False."
                        Exit Try
                    End If
                Case CnslValTypes.Integer
                    Dim num As Int32
                    If Not Int32.TryParse(sb.ToString, CI, num) Then
                        errMessage = GetMessage(3, dspMsg) ' "Value should be an integer."
                        Exit Try
                    End If
                Case CnslValTypes.Int64
                    Dim num As Int64
                    If Not Int64.TryParse(sb.ToString, CI, num) Then
                        errMessage = GetMessage(4, dspMsg) ' "Value should be an Int64."
                        Exit Try
                    End If
                Case CnslValTypes.Decimal
                    Dim num As Decimal
                    If Not Decimal.TryParse(sb.ToString, CI, num) Then
                        errMessage = GetMessage(5, dspMsg) ' "Value should be of 'Decimal' type."
                        Exit Try
                    End If
                Case CnslValTypes.Double
                    Dim num As Double
                    If Not Double.TryParse(sb.ToString, CI, num) Then
                        errMessage = GetMessage(6, dspMsg) ' "Value should be a numeric 'Double'."
                        Exit Try
                    End If
                Case CnslValTypes.Date
                    Dim num As Date
                    If Not Date.TryParse(sb.ToString, CI, num) Then
                        errMessage = GetMessage(7, dspMsg) ' "Date format is not correct."
                        Exit Try
                    End If
            End Select
            If patt.Length Then
                ret = Regex.IsMatch(sb.ToString, patt)
                If Not ret Then
                    errMessage = GetMessage(8, dspMsg) ' "Value does not match the pattern."
                End If
            Else
                ret = True
            End If
        Catch ex As Exception
            Throw
        End Try
        Return ret
    End Function

    Public Shared Sub DisplayMsg(msg As String, Optional clr As ConsoleColor = ConsoleColor.Red, Optional incrLineNumber As Int32 = 0)
        Console.ForegroundColor = clr
        Cnsl.cnslWriteFromLeftZero("", 0, msg, True)
        Console.ForegroundColor = ConsoleColor.White
        Console.CursorTop += incrLineNumber
    End Sub
    Public Shared Function GetMessage(msgNum As Int32, dsp As Boolean, ParamArray params() As Object) As String
        Dim sRet As String = errMessages(msgNum)
        If params IsNot Nothing Then
            sRet = String.Format(sRet, params)
        End If
        If dsp Then DisplayMsg(sRet)
        Return sRet
    End Function
End Class
Public Enum CnslValTypes
    [Boolean]
    [Integer]
    [Int64]
    [Decimal]
    [Double]
    [String]
    [Date]
    Password
    RagexPattern
End Enum
Public Class Cnsl
    Public Shared passwordMask As Char = "*"
    Shared lstLines As New List(Of String)
    Shared currLstLines As Int32
    Shared sb As StringBuilder
    Friend Shared topLastMsg As Int32
    Shared top As Int32
    Public Shared Function ReadLine(strToWrite As String,
                      intercept As Boolean,
                      Optional saveInput As Boolean = True,
                      Optional defaultValue As String = "",
                      Optional minLength As Int32 = 1,
                      Optional validation As CnslValidation = Nothing) As String

        If Not intercept Then Return Console.ReadLine
        If validation Is Nothing Then
            validation = New CnslValidation(CnslValTypes.String, True, minLength:=minLength)
        End If
        currLstLines = lstLines.Count - 1
        strToWrite = strToWrite.Replace(vbCr, "").Replace(vbLf, "")
        sb = New StringBuilder(defaultValue)
        cnslWriteFromLeftZero(strToWrite, 0, defaultValue)
        top = Console.CursorTop
        Dim position As Int32 = 0
        Dim sblen As Int32 = sb.Length
        Dim c As ConsoleKeyInfo
        Dim validated As Boolean
        Dim bDspLenMsg As Boolean = True
        topLastMsg = -1
        If minLength < 0 Then
            bDspLenMsg = False : minLength = -minLength
        End If
        Do
            Do
                Console.CursorLeft = strToWrite.Length + position
                c = Console.ReadKey(True)
                If Not Char.IsControl(c.KeyChar) Then
                    sb.Insert(position, c.KeyChar)
                    position += 1
                Else
                    Select Case c.Key
                        Case ConsoleKey.Delete
                            If position < sb.Length Then
                                sb.Remove(position, 1)
                            End If
                        Case ConsoleKey.Backspace
                            If position Then
                                position -= 1
                                sb.Remove(position, 1)
                            End If
                        Case ConsoleKey.LeftArrow
                            position = Math.Max(position - 1, 0)
                        Case ConsoleKey.RightArrow
                            position = Math.Min(position + 1, sb.Length)
                        Case ConsoleKey.UpArrow
                            If currLstLines >= lstLines.Count Then
                                currLstLines = lstLines.Count - 2
                            End If
                            If currLstLines >= 0 AndAlso currLstLines < lstLines.Count Then
                                sb.Clear()
                                sb.Append(lstLines(currLstLines))
                                position = sb.Length
                            End If
                            currLstLines -= 1
                        Case ConsoleKey.DownArrow
                            If currLstLines < 0 Then
                                currLstLines = 1
                            End If
                            If currLstLines >= 0 AndAlso currLstLines < lstLines.Count Then
                                sb.Clear()
                                sb.Append(lstLines(currLstLines))
                                position = sb.Length
                                If currLstLines < lstLines.Count - 1 Then
                                End If
                            End If
                            currLstLines += 1
                        Case ConsoleKey.End
                            position = sb.Length
                        Case ConsoleKey.Home
                            position = 0
                        Case Else
                            For i As Int32 = 0 To CnslFns.functionKeys.Count - 1
                                Dim val As String = CnslFns.functionKeys.Values(i)
                                Dim m As Match = Regex.Match(val, "(S)(C)(A)")
                                'Dim n As Int32 = Int32.Parse(val.Substring(1, 3))
                                Dim sSuccess As New StringBuilder("111")
                                If m.Groups(0).Success Then
                                    If (ConsoleModifiers.Shift And c.Modifiers) = 0 Then sSuccess.Chars(0) = "0"
                                End If
                                If m.Groups(1).Success Then
                                    If (ConsoleModifiers.Control And c.Modifiers) = 0 Then sSuccess.Chars(1) = "0"
                                End If
                                If m.Groups(2).Success Then
                                    If (ConsoleModifiers.Alt And c.Modifiers) = 0 Then sSuccess.Chars(2) = "0"
                                End If
                                If sSuccess.ToString = "111" Then
                                    Select Case c.Key
                                        Case ConsoleKey.F1
                                            sb = New StringBuilder(CnslFns.functionKeys("DspHelp"))
                                            Exit Do
                                        Case ConsoleKey.F2
                                            sb = New StringBuilder(CnslFns.functionKeys("AskAgain"))
                                            Exit Do
                                        Case ConsoleKey.F3
                                            sb = New StringBuilder(CnslFns.functionKeys("Examples"))
                                            Exit Do
                                        Case ConsoleKey.F4
                                            Console.Clear()
                                            top = 0 : position = 0
                                            sb = New StringBuilder(CnslFns.functionKeys("CLS"))
                                            Exit Do
                                        Case ConsoleKey.F5
                                            sb = New StringBuilder(CnslFns.functionKeys("Quit"))
                                            Exit Do
                                    End Select
                                End If
                            Next
                            Select Case c.Key
                                Case ConsoleKey.Escape
                                    position = 0
                                    sb.Clear()
                                Case ConsoleKey.Enter
                                Case Else

                                    Dim IsTest As Boolean = False ' Set to true for test purposses: 
                                    If IsTest Then
                                        CnslValidation.DisplayMsg("Not implemented. You pressed " +
                                                     IIf(c.Modifiers And ConsoleModifiers.Shift, "Shift+", "") +
                                                     IIf(c.Modifiers And ConsoleModifiers.Control, "Control+", "") +
                                                     IIf(c.Modifiers And ConsoleModifiers.Alt, "Alt+", "") + c.Key.ToString)
                                        Exit Do
                                    End If
                            End Select
                    End Select
                    cnslWriteFromLeftZero(strToWrite, position, sb.ToString)
                End If

                sblen = sb.Length
                If sb.Length Then
                    If validation.inpType = CnslValTypes.Password Then
                        Dim sInput As String = StrDup(sb.Length, passwordMask)
                        cnslWriteFromLeftZero(strToWrite, position, sInput)
                    Else
                        cnslWriteFromLeftZero(strToWrite, position, sb.ToString)
                    End If
                End If
            Loop While c.Key <> ConsoleKey.Enter OrElse Not CnslFns.IsNotAFunctKey(sb.ToString)

            If CnslFns.IsNotAFunctKey(sb.ToString) Then

                validation.maxNumOfTries -= 1

                Dim topCpy As Int32 = top + 1
                setCursorTop(topCpy)
                Console.CursorLeft = 0
                validated = False
                If validation.maxNumOfTries < 0 Then
                    sb = New StringBuilder(CnslFns.functionKeys("TriesExahusted"))
                Else
                    validated = validation.Validate(sb)
                    If Not validated Then
                        setCursorTop(top)
                        Console.CursorLeft = strToWrite.Length + position
                        Continue Do
                    End If
                    If validation.callBk IsNot Nothing Then
                        topCpy = top + IIf(top + 1 < Console.BufferHeight, 1, 0)
                        setCursorTop(topCpy)
                        Console.CursorLeft = 0
                        Try
                            validated = validation.callBk(sb, validation)
                            If Not validated Then
                                setCursorTop(top)
                                Console.CursorLeft = strToWrite.Length + position
                                Continue Do
                            End If
                        Catch ex As Exception
                            sb = New StringBuilder(CnslFns.functionKeys("Quit"))
                        End Try
                    End If
                End If
            Else
                validated = True
            End If


            If top + 1 < Console.BufferHeight Then top += 1
            setCursorTop(top)
            Console.CursorLeft = 0
            If saveInput AndAlso Trim(sb.ToString).Length AndAlso
               CnslFns.IsNotAFunctKey(sb.ToString) AndAlso Not validation.inpType = CnslValTypes.Password Then
                lstLines.Add(sb.ToString)
            End If
        Loop While Not validated AndAlso validation.maxNumOfTries >= 0
        Return sb.ToString
    End Function
    Shared Sub setCursorTop(ByRef currTop As Int32)
        If currTop >= Console.BufferHeight Then
            currTop = Console.BufferHeight - 1
            Console.WriteLine("")
        End If
        Console.CursorTop = currTop
    End Sub
    Shared Sub cnslWriteFromLeftZero(strToWrite As String, pos As Int32, sInput As String, Optional IsErrMsg As Boolean = False)
        Console.CursorLeft = 0
        Console.Write(StrDup(79, " "))
        Console.CursorLeft = 0
        Console.Write(strToWrite + sInput)
        If IsErrMsg AndAlso top = Console.BufferHeight - 1 Then top = Console.BufferHeight - 2
        Console.CursorLeft = pos
    End Sub
    Shared Sub Addline(strToWrite As String, ByRef top As Int32)
        Console.WriteLine("")
        top = Console.BufferHeight - 1
        Console.Write(strToWrite + sb.ToString)
    End Sub
End Class

